:root {
  --paper: #f8f4ea;
  --paper-strong: #fffdf8;
  --ink: #132335;
  --muted: #556678;
  --border: rgba(19, 35, 53, 0.16);
  --teal: #0d8b84;
  --teal-soft: #d9f3ef;
  --navy: #12304a;
  --amber: #f2b443;
  --amber-soft: #fff0cc;
  --rose: #d86557;
  --shadow: 0 22px 60px rgba(18, 48, 74, 0.16);
  --radius-lg: 28px;
  --radius-md: 18px;
  --radius-sm: 12px;
}

html {
  color: var(--ink);
  background: linear-gradient(180deg, #f5efe2 0%, var(--paper) 35%, #fff 100%);
  -webkit-print-color-adjust: exact;
  print-color-adjust: exact;
}

body {
  margin: 0 auto;
  max-width: 8.2in;
  padding: 0.35in 0.1in 0.7in;
  font-family: Georgia, "Times New Roman", serif;
  line-height: 1.58;
  color: var(--ink);
}

a,
a:visited {
  color: var(--teal);
}

p,
li {
  font-size: 10.5pt;
}

h1,
h2,
h3 {
  font-family: "Helvetica Neue", Arial, sans-serif;
  letter-spacing: 0.01em;
  color: var(--navy);
}

h1 {
  margin: 0 0 0.18in;
  font-size: 28pt;
  line-height: 1.05;
}

h2 {
  margin: 0.55in 0 0.16in;
  font-size: 17pt;
  page-break-after: avoid;
}

h3 {
  margin: 0.28in 0 0.08in;
  font-size: 12.5pt;
}

hr {
  border: 0;
  border-top: 1px solid var(--border);
  margin: 0.32in 0;
}

code {
  font-family: "SFMono-Regular", "SF Mono", Menlo, Consolas, monospace;
  font-size: 0.92em;
  background: rgba(13, 139, 132, 0.08);
  border-radius: 6px;
  padding: 0.08em 0.34em;
}

pre {
  overflow-x: auto;
  padding: 0.18in 0.2in;
  border-radius: 18px;
  border: 1px solid rgba(255, 255, 255, 0.08);
  background:
    radial-gradient(circle at top left, rgba(242, 180, 67, 0.12), transparent 35%),
    linear-gradient(180deg, #102534 0%, #0d1c28 100%);
  color: #f7fbff;
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.06), 0 18px 40px rgba(10, 23, 35, 0.18);
  page-break-inside: avoid;
}

pre code {
  background: transparent;
  padding: 0;
  color: inherit;
}

table {
  width: 100%;
  border-collapse: collapse;
  margin: 0.18in 0 0.24in;
  border: 1px solid var(--border);
  border-radius: var(--radius-md);
  overflow: hidden;
  box-shadow: 0 10px 30px rgba(18, 48, 74, 0.08);
}

thead {
  background: linear-gradient(135deg, #173753, #0d8b84);
  color: #fff;
}

th,
td {
  padding: 0.14in 0.15in;
  border-bottom: 1px solid var(--border);
  vertical-align: top;
  text-align: left;
  font-size: 9.7pt;
}

tbody tr:nth-child(odd) {
  background: rgba(255, 255, 255, 0.7);
}

tbody tr:nth-child(even) {
  background: rgba(217, 243, 239, 0.26);
}

blockquote {
  margin: 0.24in 0;
  padding: 0.14in 0.18in;
  border-left: 4px solid var(--amber);
  background: rgba(242, 180, 67, 0.1);
  border-radius: 0 14px 14px 0;
}

img {
  max-width: 100%;
}

figure {
  margin: 0.22in 0 0.18in;
  page-break-inside: avoid;
}

figcaption {
  margin-top: 0.1in;
  font-size: 8.8pt;
  color: var(--muted);
}

.hero-shell {
  display: grid;
  grid-template-columns: minmax(0, 1.18fr) minmax(0, 0.82fr);
  gap: 0.25in;
  align-items: stretch;
  padding: 0.24in;
  border-radius: 32px;
  background:
    radial-gradient(circle at top left, rgba(242, 180, 67, 0.24), transparent 32%),
    linear-gradient(135deg, rgba(18, 48, 74, 0.96), rgba(13, 139, 132, 0.92));
  box-shadow: var(--shadow);
  color: #fff;
  page-break-inside: avoid;
}

.hero-copy {
  display: flex;
  flex-direction: column;
  justify-content: center;
}

.eyebrow {
  margin: 0 0 0.08in;
  font-family: "Helvetica Neue", Arial, sans-serif;
  font-size: 8.6pt;
  font-weight: 700;
  letter-spacing: 0.16em;
  text-transform: uppercase;
  color: var(--teal);
}

.hero-shell .eyebrow {
  color: rgba(255, 255, 255, 0.78);
}

.hero-deck {
  margin: 0;
  font-family: "Helvetica Neue", Arial, sans-serif;
  font-size: 16pt;
  line-height: 1.3;
  font-weight: 600;
}

.hero-art {
  align-self: center;
  border-radius: 24px;
  background: rgba(255, 255, 255, 0.08);
}

.pill-row {
  display: flex;
  flex-wrap: wrap;
  gap: 0.08in;
  margin-top: 0.2in;
}

.pill {
  display: inline-flex;
  align-items: center;
  padding: 0.06in 0.12in;
  border-radius: 999px;
  border: 1px solid rgba(255, 255, 255, 0.18);
  background: rgba(255, 255, 255, 0.09);
  font-family: "Helvetica Neue", Arial, sans-serif;
  font-size: 8.8pt;
  font-weight: 600;
}

.callout-grid {
  display: grid;
  grid-template-columns: repeat(2, minmax(0, 1fr));
  gap: 0.18in;
  margin: 0.28in 0 0.3in;
}

.callout {
  padding: 0.18in 0.2in;
  border-radius: var(--radius-lg);
  background: rgba(255, 255, 255, 0.78);
  border: 1px solid var(--border);
  box-shadow: 0 14px 34px rgba(18, 48, 74, 0.09);
  break-inside: avoid;
  page-break-inside: avoid;
}

.callout ul {
  margin: 0.08in 0 0;
  padding-left: 0.22in;
}

.callout p:last-child {
  margin-bottom: 0;
}

.callout .eyebrow {
  color: var(--navy);
}

.callout.accent {
  background: linear-gradient(180deg, rgba(255, 240, 204, 0.96), rgba(255, 255, 255, 0.92));
}

.callout.accent .eyebrow {
  color: #8c5a00;
}

.section-divider {
  display: block;
  width: 100%;
  margin: 0.08in 0 0.22in;
}

.key-grid {
  display: grid;
  grid-template-columns: repeat(3, minmax(0, 1fr));
  gap: 0.16in;
  margin: 0.18in 0 0.26in;
}

.key-card {
  padding: 0.16in 0.18in;
  border-radius: 22px;
  border: 1px solid rgba(18, 48, 74, 0.12);
  background:
    radial-gradient(circle at top right, rgba(242, 180, 67, 0.16), transparent 38%),
    rgba(255, 255, 255, 0.9);
  box-shadow: 0 14px 30px rgba(18, 48, 74, 0.08);
  break-inside: avoid;
  page-break-inside: avoid;
}

.key-card p {
  margin: 0;
}

.key-card .eyebrow {
  margin-bottom: 0.12in;
  color: var(--navy);
  letter-spacing: 0.08em;
}

.key-card p + .key-label,
.key-card p + p:not(.key-label) {
  margin-top: 0.08in;
}

.key-label {
  font-family: "Helvetica Neue", Arial, sans-serif;
  font-size: 8.4pt;
  font-weight: 700;
  letter-spacing: 0.08em;
  text-transform: uppercase;
  color: var(--muted);
}

.figure-grid {
  display: grid;
  gap: 0.18in;
  margin: 0.14in 0 0.24in;
}

.figure-grid.two-up {
  grid-template-columns: repeat(2, minmax(0, 1fr));
}

.figure-grid.quad {
  grid-template-columns: repeat(2, minmax(0, 1fr));
}

.shot {
  margin: 0;
  padding: 0.12in;
  border-radius: 22px;
  background: rgba(255, 255, 255, 0.88);
  border: 1px solid rgba(18, 48, 74, 0.1);
  box-shadow: 0 18px 34px rgba(18, 48, 74, 0.12);
  break-inside: avoid;
  page-break-inside: avoid;
}

.shot img {
  display: block;
  width: 100%;
  border-radius: 14px;
}

.shot.wide {
  padding: 0.14in;
}

ul,
ol {
  margin-top: 0.08in;
}

li + li {
  margin-top: 0.04in;
}

@page {
  size: Letter;
  margin: 0.5in 0.55in 0.6in;
}

@media print {
  body {
    max-width: none;
    padding: 0;
  }

  a {
    text-decoration: none;
  }

  .callout-grid,
  .key-grid,
  .figure-grid.two-up,
  .figure-grid.quad {
    display: block;
  }

  .callout,
  .key-card,
  .shot {
    margin-bottom: 0.18in;
  }

  h2,
  h3,
  pre,
  figure,
  table,
  .callout,
  .hero-shell {
    page-break-inside: avoid;
  }
}
